#!/usr/bin/env python3
# This file is placed in the Public Domain.


import threading
import time


from .obj import find, fntime, listfiles
from .obj import Cfg, Object, Table, get, keys, save, update
from .obj import fmt, getname
from .run import Bus, elapsed, starttime


## defines


def init():
    pass


class Log(Object):

    def __init__(self):
        super().__init__()
        self.txt = ""


## commands


def cmd(event):
    event.reply(",".join(sorted(Table.cmds)))


def flt(event):
    try:
        index = int(event.args[0])
        event.reply(fmt(Bus.objs[index], skip=["queue", "ready", "iqueue"]))
        return
    except (TypeError, IndexError, ValueError):
        pass
    event.reply(" | ".join([getname(o) for o in Bus.objs]))


def fnd(event):
    if not event.args:
        fls = listfiles(Cfg.wd)
        if fls:
            event.reply(",".join(sorted({x.split(".")[-1].lower()
                        for x in fls})))
        return
    otype = event.args[0]
    nr = -1
    if "a" in event.opts:
        args = None
    else:
        args = list(event.gets)
        try:
            args.extend(event.args[1:])
        except IndexError:
            pass
    got = False
    for fn, o in find(otype, event.gets, event.index, event.timed):
        nr += 1
        txt = "%s %s" % (str(nr), fmt(o, args or keys(o),
                                      skip=keys(event.skip)))
        if "t" in event.opts:
            txt = txt + " %s" % (elapsed(time.time() - fntime(fn)))
        got = True
        event.reply(txt)
    if not got:
        event.reply("no result")


def log(event):
    if not event.rest:
        event.reply("log <txt>")
        return
    o = Log()
    o.txt = event.rest
    save(o)
    event.reply("ok")


def thr(event):
    result = []
    for t in sorted(threading.enumerate(), key=lambda x: x.getName()):
        if str(t).startswith("<_"):
            continue
        o = Object()
        update(o, vars(t))
        if get(o, "sleep", None):
            up = o.sleep - int(time.time() - o.state.latest)
        else:
            up = int(time.time() - starttime)
        thrname = t.getName()
        if not thrname:
            continue
        if thrname:
            result.append((up, thrname))
    res = []
    for up, txt in sorted(result, key=lambda x: x[0]):
        res.append("%s(%s)" % (txt, elapsed(up)))
    if res:
        event.reply(" ".join(res))


def upt(event):
    event.reply(elapsed(time.time() - starttime))
